@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2010-2023 darktable developers.
+    Copyright (C) 2010-2025 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -89,6 +89,11 @@ typedef struct dt_variables_data_t
   float exif_focal_length;
   float exif_crop;
   float exif_focus_distance;
+  const char *exif_flash;
+  const char *exif_flash_icon;
+  const char *exif_exposure_program;
+  const char *exif_metering_mode;
+  const char *exif_whitebalance;
   double longitude;
   double latitude;
   double elevation;
@@ -101,6 +106,7 @@ typedef struct dt_variables_data_t
 
 static char *_expand_source(dt_variables_params_t *params, char **source, char extra_stop);
 
+
 // gather some data that might be used for variable expansion
 static void _init_expansion(dt_variables_params_t *params, gboolean iterate)
 {
@@ -136,17 +142,22 @@ static void _init_expansion(dt_variables_params_t *params, gboolean iterate)
   params->data->exif_focal_length = 0.0f;
   params->data->exif_crop = 0.0f;
   params->data->exif_focus_distance = 0.0f;
+  params->data->exif_flash = "";
+  params->data->exif_flash_icon = "";
+  params->data->exif_exposure_program = "";
+  params->data->exif_metering_mode = "";
+  params->data->exif_whitebalance = "";
   params->data->longitude = NAN;
   params->data->latitude = NAN;
   params->data->elevation = NAN;
   params->data->show_msec = dt_conf_get_bool("lighttable/ui/milliseconds");
+  params->data->camera_maker = NULL;
+  params->data->camera_alias = NULL;
   if(dt_is_valid_imgid(params->imgid))
   {
-    params->data->camera_maker = NULL;
-    params->data->camera_alias = NULL;
     const dt_image_t *img = params->img
       ? (dt_image_t *)params->img
-      : dt_image_cache_get(darktable.image_cache, params->imgid, 'r');
+      : dt_image_cache_get(params->imgid, 'r');
 
     params->data->datetime = dt_datetime_img_to_gdatetime(img, darktable.utc_tz);
     if(params->data->datetime)
@@ -171,6 +182,25 @@ static void _init_expansion(dt_variables_params_t *params, gboolean iterate)
     params->data->latitude = img->geoloc.latitude;
     params->data->elevation = img->geoloc.elevation;
 
+
+    // We don't want to claim that the flash did not fire when the photo is
+    // clearly taken with a flash, but information about this is not available
+    if(img->exif_flash_tagvalue == -1)
+    {
+       params->data->exif_flash_icon = "";
+       params->data->exif_flash = _("no info");
+    }
+    else
+    {
+      // Bit 0 set means that flash was fired
+      params->data->exif_flash_icon = (img->exif_flash_tagvalue & 1) ? "⚡" : "";
+      params->data->exif_flash = (img->exif_flash_tagvalue & 1) ? _("yes") : _("no");
+    }
+
+    params->data->exif_exposure_program = img->exif_exposure_program;
+    params->data->exif_metering_mode = img->exif_metering_mode;
+    params->data->exif_whitebalance = img->exif_whitebalance;
+
     params->data->flags = img->flags;
 
     params->data->raw_height = img->p_height;
@@ -199,15 +229,15 @@ static void _init_expansion(dt_variables_params_t *params, gboolean iterate)
       }
     }
 
-    if(params->img == NULL) dt_image_cache_read_release(darktable.image_cache, img);
+    if(params->img == NULL) dt_image_cache_read_release(img);
   }
   else
   { // session data
     params->data->datetime = params->data->exif_time;
     if(params->data->datetime)
       params->data->have_exif_dt = TRUE;
-    params->data->camera_maker = params->data->exif_maker;
-    params->data->camera_alias = params->data->exif_model;
+    params->data->camera_maker = g_strdup(params->data->exif_maker);
+    params->data->camera_alias = g_strdup(params->data->exif_model);
   }
 }
 
@@ -221,10 +251,14 @@ static void _cleanup_expansion(dt_variables_params_t *params)
       params->data->datetime = NULL;
     }
     g_free(params->data->camera_maker);
+    params->data->camera_maker = NULL;
     g_free(params->data->camera_alias);
+    params->data->camera_alias = NULL;
   }
   g_free(params->data->homedir);
+  params->data->homedir = NULL;
   g_free(params->data->pictures_folder);
+  params->data->pictures_folder = NULL;
 }
 
 static inline gboolean _has_prefix(char **str, const char *prefix)
@@ -266,6 +300,56 @@ static char *_variables_get_latitude(dt_variables_params_t *params)
   }
 }
 
+/**
+ * get n,m from a variable with [n,m], i.e. $(SEQUENCE[4,1])
+ * param_m may be NULL if only n is needed, i.e. $(IMAGE_ID[4])
+ */
+static void _get_parameters_n_m(char **variable, gchar **param_n, gchar **param_m)
+{
+  *param_n = NULL;
+  if(param_m) *param_m = NULL;
+
+  if(*variable[0] != '[') return;
+  (*variable)++;
+  if(*variable[0] == ',') return;
+
+  gchar *parameters = g_strdup(*variable);
+  char *end = g_strstr_len(parameters, -1, "]");
+  if(end)
+  {
+    end[0] = '\0';
+    (*variable) += strlen(parameters) + 1;
+
+    gchar **res = g_strsplit(parameters, ",", 2);
+    gchar **p = res;
+    int n = 0;
+    while(*p && n < 2)
+    {
+      gchar **target = (n == 0)? param_n : param_m;
+      if(target) *target = g_strdup(*p);
+      p++;
+      n++;
+    }
+    g_strfreev(res);
+  }
+  g_free(parameters);
+}
+
+static uint8_t _get_var_parameter(char **variable, const int default_value)
+{
+  uint8_t val = default_value;
+  if(*variable[0] == '[')
+  {
+    gchar *val_s;
+    _get_parameters_n_m(variable, &val_s, NULL);
+    int n = g_ascii_strtoll(val_s, NULL, 10);
+    if(n > 0)
+      val = n;
+    g_free(val_s);
+  }
+  return val;
+}
+
 static char *_get_base_value(dt_variables_params_t *params, char **variable)
 {
   char *result = NULL;
@@ -383,13 +467,22 @@ static char *_get_base_value(dt_variables_params_t *params, char **variable)
   else if(_has_prefix(variable, "EXIF.ISO") || _has_prefix(variable, "EXIF_ISO"))
     result = g_strdup_printf("%d", params->data->exif_iso);
   else if(_has_prefix(variable, "NL") && g_strcmp0(params->jobcode, "infos") == 0)
-    result = g_strdup_printf("\n");
+  {
+    if(params->use_html_newline)
+      result = g_strdup_printf("&#13;");
+    else
+      result = g_strdup_printf("\n");
+  }
   else if(_has_prefix(variable, "EXIF.EXPOSURE.BIAS")
           || _has_prefix(variable, "EXIF_EXPOSURE_BIAS"))
   {
     if(params->data->exif_exposure_bias != DT_EXIF_TAG_UNINITIALIZED)
       result = g_strdup_printf("%+.2f", params->data->exif_exposure_bias);
   }
+  else if(_has_prefix(variable, "EXIF.EXPOSURE.PROGRAM"))
+  {
+    result = g_strdup(params->data->exif_exposure_program);
+  }
   else if(_has_prefix(variable, "EXIF.EXPOSURE")
           || _has_prefix(variable, "EXIF_EXPOSURE"))
   {
@@ -420,6 +513,18 @@ static char *_get_base_value(dt_variables_params_t *params, char **variable)
           || _has_prefix(variable, "EXIF_FOCUS_DISTANCE"))
     result = g_strdup_printf("%.2f", params->data->exif_focus_distance);
 
+  else if(_has_prefix(variable, "EXIF.FLASH.ICON"))
+    result = g_strdup(params->data->exif_flash_icon);
+
+  else if(_has_prefix(variable, "EXIF.FLASH"))
+    result = g_strdup(params->data->exif_flash);
+
+  else if(_has_prefix(variable, "EXIF.METERING"))
+    result = g_strdup(params->data->exif_metering_mode);
+
+  else if(_has_prefix(variable, "EXIF.WHITEBALANCE"))
+    result = g_strdup(params->data->exif_whitebalance);
+
   else if(_has_prefix(variable, "LONGITUDE")
           || _has_prefix(variable, "GPS.LONGITUDE"))
     result = _variables_get_longitude(params);
@@ -432,6 +537,11 @@ static char *_get_base_value(dt_variables_params_t *params, char **variable)
           || _has_prefix(variable, "GPS.ELEVATION"))
     result = g_strdup_printf("%.2f", params->data->elevation);
 
+  else if(_has_prefix(variable, "GPS.LOCATION.ICON"))
+    // possible symbols: world map U+1F5FA 🗺, globe w/ meridians U+1F310 🌐
+    //   pushpin U+1F4CC 📌 round pushpin U+1F4CD 📍   black pushpin U+1F588 🖈
+    result = g_strdup((!isnan(params->data->latitude) && !isnan(params->data->longitude)) ? "📌" : "");
+
   // for watermark backward compatibility
   else if(_has_prefix(variable, "GPS.LOCATION"))
   {
@@ -454,24 +564,46 @@ static char *_get_base_value(dt_variables_params_t *params, char **variable)
     result = g_strdup(params->data->camera_alias);
   else if(_has_prefix(variable, "EXIF.LENS") || _has_prefix(variable, "LENS"))
     result = g_strdup(params->data->exif_lens);
+  else if(_has_prefix(variable, "IMAGE.ID.NEXT"))
+  {
+    dt_imgid_t highest_id = 0;
+    sqlite3_stmt *stmt;
+    // clang-format off
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                                "SELECT MAX(id) FROM main.images",
+                                -1, &stmt, NULL);
+    // clang-format on
+    if(sqlite3_step(stmt) == SQLITE_ROW)
+      highest_id = sqlite3_column_int(stmt, 0);
+    sqlite3_finalize(stmt);
+    // determine how many zero-padded digits to use with an optional
+    // parameter: $(IMAGE.ID.NEXT[n]), default n=1
+    uint8_t nb_digit = _get_var_parameter(variable, 1);
+    result = g_strdup_printf("%0*u", nb_digit, highest_id + 1);
+  }
   else if(_has_prefix(variable, "ID") || _has_prefix(variable, "IMAGE.ID"))
-    result = g_strdup_printf("%u", params->imgid);
+  {
+    // determine how many zero-padded digits to use with an optional
+    // parameter: $(IMAGE.ID[n]), default n=1
+    uint8_t nb_digit = _get_var_parameter(variable, 1);
+    result = g_strdup_printf("%0*u", nb_digit, params->imgid);
+  }
   else if(_has_prefix(variable, "IMAGE.EXIF"))
   {
     gchar buffer[1024];
     const dt_image_t *img = params->img
       ? (dt_image_t *)params->img
-      : dt_image_cache_get(darktable.image_cache, params->imgid, 'r');
+      : dt_image_cache_get(params->imgid, 'r');
 
     dt_image_print_exif(img, buffer, sizeof(buffer));
     if(params->img == NULL)
-      dt_image_cache_read_release(darktable.image_cache, img);
+      dt_image_cache_read_release(img);
     result = g_strdup(buffer);
   }
   else if(_has_prefix(variable, "VERSION.NAME")
           || _has_prefix(variable, "VERSION_NAME"))
   {
-    GList *res = dt_metadata_get(params->imgid, "Xmp.darktable.version_name", NULL);
+    GList *res = dt_metadata_get_lock(params->imgid, "Xmp.darktable.version_name", NULL);
     if(res != NULL)
     {
       result = g_strdup((char *)res->data);
@@ -555,15 +687,40 @@ static char *_get_base_value(dt_variables_params_t *params, char **variable)
   else if(_has_prefix(variable, "SEQUENCE"))
   {
     uint8_t nb_digit = 4;
+    guint shift = 1;
+    // old $(SEQUENCEn) syntax for backwards compatibility
     if(g_ascii_isdigit(*variable[0]))
     {
       nb_digit = (uint8_t)*variable[0] & 0b1111;
-      (*variable) ++;
+      (*variable)++;
+    }
+    // new $(SEQUENCE[n,m]) syntax
+    // allows \[[0-9]+,[0-9]+] (PCRE)
+    // everything else will be ignored
+    else if(*variable[0] == '[')
+    {
+      gchar *nb_digit_s, *shift_s;
+      _get_parameters_n_m(variable, &nb_digit_s, &shift_s);
+
+      if(nb_digit_s)
+      {
+        int nb = g_ascii_strtoll(nb_digit_s, NULL, 10);
+        if(nb > 0) nb_digit = nb;
+      }
+
+      if(shift_s)
+      {
+        int nb = g_ascii_strtoll(shift_s, NULL, 10);
+        if(nb > 0) shift = nb;
+      }
+
+      g_free(nb_digit_s);
+      g_free(shift_s);
     }
     result = g_strdup_printf("%.*u", nb_digit,
                              params->sequence >= 0
-                             ? params->sequence
-                             : params->data->sequence);
+                             ? params->sequence + shift - 1
+                             : params->data->sequence + shift - 1);
   }
   else if(_has_prefix(variable, "USERNAME"))
     result = g_strdup(g_get_user_name());
@@ -622,13 +779,13 @@ static char *_get_base_value(dt_variables_params_t *params, char **variable)
           && g_strcmp0(params->jobcode, "infos") == 0)
   {
     escape = FALSE;
-    GList *res = dt_metadata_get(params->imgid, "Xmp.darktable.colorlabels", NULL);
+    GList *res = dt_metadata_get_lock(params->imgid, "Xmp.darktable.colorlabels", NULL);
     for(GList *res_iter = res; res_iter; res_iter = g_list_next(res_iter))
     {
       const int dot_index = GPOINTER_TO_INT(res_iter->data);
       const GdkRGBA c = darktable.bauhaus->colorlabels[dot_index];
-      result = dt_util_dstrcat
-        (result,
+      dt_util_str_cat
+        (&result,
          "<span foreground='#%02x%02x%02x'>⬤ </span>",
          (guint)(c.red*255), (guint)(c.green*255), (guint)(c.blue*255));
     }
@@ -639,7 +796,7 @@ static char *_get_base_value(dt_variables_params_t *params, char **variable)
     // TODO: currently we concatenate all the color labels with a ','
     // as a separator. Maybe it's better to only use the first/last
     // label?
-    GList *res = dt_metadata_get(params->imgid, "Xmp.darktable.colorlabels", NULL);
+    GList *res = dt_metadata_get_lock(params->imgid, "Xmp.darktable.colorlabels", NULL);
     if(res != NULL)
     {
       GList *labels = NULL;
@@ -655,56 +812,6 @@ static char *_get_base_value(dt_variables_params_t *params, char **variable)
     }
     g_list_free(res);
   }
-  else if(_has_prefix(variable, "TITLE")
-          || _has_prefix(variable, "Xmp.dc.title"))
-  {
-    GList *res = dt_metadata_get(params->imgid, "Xmp.dc.title", NULL);
-    if(res != NULL)
-    {
-      result = g_strdup((char *)res->data);
-    }
-    g_list_free_full(res, &g_free);
-  }
-  else if(_has_prefix(variable, "DESCRIPTION")
-          || _has_prefix(variable, "Xmp.dc.description"))
-  {
-    GList *res = dt_metadata_get(params->imgid, "Xmp.dc.description", NULL);
-    if(res != NULL)
-    {
-      result = g_strdup((char *)res->data);
-    }
-    g_list_free_full(res, &g_free);
-  }
-  else if(_has_prefix(variable, "CREATOR")
-          || _has_prefix(variable, "Xmp.dc.creator"))
-  {
-    GList *res = dt_metadata_get(params->imgid, "Xmp.dc.creator", NULL);
-    if(res != NULL)
-    {
-      result = g_strdup((char *)res->data);
-    }
-    g_list_free_full(res, &g_free);
-  }
-  else if(_has_prefix(variable, "PUBLISHER")
-          || _has_prefix(variable, "Xmp.dc.publisher"))
-  {
-    GList *res = dt_metadata_get(params->imgid, "Xmp.dc.publisher", NULL);
-    if(res != NULL)
-    {
-      result = g_strdup((char *)res->data);
-    }
-    g_list_free_full(res, &g_free);
-  }
-  else if(_has_prefix(variable, "RIGHTS")
-          || _has_prefix(variable, "Xmp.dc.rights"))
-  {
-    GList *res = dt_metadata_get(params->imgid, "Xmp.dc.rights", NULL);
-    if(res != NULL)
-    {
-      result = g_strdup((char *)res->data);
-    }
-    g_list_free_full(res, &g_free);
-  }
   else if(_has_prefix(variable, "OPENCL.ACTIVATED")
           || _has_prefix(variable, "OPENCL_ACTIVATED"))
   {
@@ -760,11 +867,12 @@ static char *_get_base_value(dt_variables_params_t *params, char **variable)
     if(g_ascii_isdigit(*variable[0]))
     {
       const uint8_t level = (uint8_t)*variable[0] & 0b1111;
-      (*variable) ++;
+      (*variable)++;
       if(*variable[0] == '(')
       {
         char *category = g_strdup(*variable + 1);
-        char *end = g_strstr_len(category, -1, ")");
+        char *end = NULL;
+        end = g_strstr_len(category, -1, ")");
         if(end)
         {
           end[0] = '|';
@@ -780,6 +888,33 @@ static char *_get_base_value(dt_variables_params_t *params, char **variable)
         g_free(category);
       }
     }
+    // $(CATEGORY[n,category]) with category as above (new syntax)
+    else if(*variable[0] == '[')
+    {
+      gchar *level_s, *category;
+      _get_parameters_n_m(variable, &level_s, &category);
+
+      if(level_s && category && g_ascii_isdigit(*level_s))
+      {
+        const uint8_t level = (uint8_t)*level_s & 0b1111;
+        gchar *cat = g_strdup_printf("%s|", category);
+        char *tag = dt_tag_get_subtags(params->imgid, cat, (int)level);
+        g_free(cat);
+        if(tag)
+        {
+          result = g_strdup(tag);
+          g_free(tag);
+        }
+      }
+      g_free(level_s);
+      g_free(category);
+    }
+  }
+  else if(_has_prefix(variable, "IMAGE.TAGS.HIERARCHY"))
+  {
+    GList *tags_list = dt_tag_get_hierarchical_export(params->imgid, params->data->tags_flags);
+    result = dt_util_glist_to_str(", ", tags_list);
+    g_list_free_full(tags_list, g_free);
   }
   else if(_has_prefix(variable, "TAGS") || _has_prefix(variable, "IMAGE.TAGS"))
   {
@@ -812,6 +947,29 @@ static char *_get_base_value(dt_variables_params_t *params, char **variable)
           || _has_prefix(variable, "DARKTABLE_NAME"))
     result = g_strdup(PACKAGE_NAME);
   else
+  {
+    // metadata
+    dt_pthread_mutex_lock(&darktable.metadata_threadsafe);
+    for(GList* iter = dt_metadata_get_list(); iter; iter = iter->next)
+    {
+      dt_metadata_t *metadata = (dt_metadata_t *)iter->data;
+      gchar *prefix = g_utf8_strup(dt_metadata_get_tag_subkey(metadata->tagname), -1);
+      gboolean found = FALSE;
+      if(_has_prefix(variable, prefix))
+      {
+        GList *res = dt_metadata_get(params->imgid, metadata->tagname, NULL);
+        if(res != NULL)
+          	result = g_strdup((char *)res->data);
+        g_list_free_full(res, g_free);
+        found = TRUE;
+      }
+      g_free(prefix);
+      if(found) break;
+    }
+    dt_pthread_mutex_unlock(&darktable.metadata_threadsafe);
+  }
+  
+  if(!result)
   {
     // go past what looks like an invalid variable. we only expect to
     // see [a-zA-Z]* in a variable name.
@@ -1201,7 +1359,6 @@ char *dt_variables_expand(dt_variables_params_t *params,
   char *result = _expand_source(params, &source, '\0');
 
   _cleanup_expansion(params);
-
   return result;
 }
 
@@ -1223,6 +1380,9 @@ void dt_variables_params_destroy(dt_variables_params_t *params)
     g_date_time_unref(params->data->exif_time);
   g_free(params->data->exif_maker);
   g_free(params->data->exif_model);
+  g_free(params->data->exif_lens);
+  g_free(params->data->camera_maker);
+  g_free(params->data->camera_alias);
   g_free(params->data);
   g_free(params);
 }
