@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2018-2023 darktable developers.
+    Copyright (C) 2018-2024 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 */
 
 #include "bauhaus/bauhaus.h"
+#include "common/curl_tools.h"
 #include "common/darktable.h"
 #include "common/file_location.h"
 #include "common/image.h"
@@ -24,11 +25,12 @@
 #include "common/metadata.h"
 #include "common/pwstorage/pwstorage.h"
 #include "common/tags.h"
-#include "common/curl_tools.h"
+#include "common/variables.h"
 #include "control/conf.h"
 #include "control/control.h"
 #include "dtgtk/button.h"
 #include "gui/gtk.h"
+#include "gui/gtkentry.h"
 #include "gui/accelerators.h"
 #include "imageio/imageio_common.h"
 #include "imageio/imageio_module.h"
@@ -40,11 +42,12 @@
 #include <unistd.h>
 #include <inttypes.h>
 
-DT_MODULE(1)
+DT_MODULE(3)
 
 #define piwigo_EXTRA_VERBOSE FALSE
 
 #define MAX_ALBUM_NAME_SIZE 100
+#define MAX_SERVER_NAME_SIZE 2048
 
 typedef struct _piwigo_api_context_t
 {
@@ -88,6 +91,7 @@ typedef struct dt_storage_piwigo_gui_data_t
   GtkWidget *album_list, *parent_album_list;
   GtkWidget *conflict_action;
   GtkWidget *account_list;
+  GtkEntry *filename_pattern_entry;
 
   GList *albums;
   GList *accounts;
@@ -102,6 +106,15 @@ typedef struct _curl_args_t
   char value[512];
 } _curl_args_t;
 
+typedef enum dt_storage_piwigo_permissions_t
+{
+  DT_PIWIGO_PERMISSION_EVERYONE = 0,
+  DT_PIWIGO_PERMISSION_CONTACTS = 1,
+  DT_PIWIGO_PERMISSION_FRIENDS = 2,
+  DT_PIWIGO_PERMISSION_FAMILY = 4,
+  DT_PIWIGO_PERMISSION_ADMIN = 8
+} dt_storage_piwigo_permissions_t;
+
 typedef enum dt_storage_piwigo_conflict_actions_t
 {
   DT_PIWIGO_CONFLICT_NOTHING = 0,
@@ -110,20 +123,82 @@ typedef enum dt_storage_piwigo_conflict_actions_t
   DT_PIWIGO_CONFLICT_OVERWRITE = 3
 } dt_storage_piwigo_conflict_actions_t;
 
+// saved params
+typedef struct dt_storage_piwigo_preset_data_t
+{
+  char filename_pattern[DT_MAX_PATH_FOR_PARAMS];
+  dt_storage_piwigo_permissions_t privacy;
+  dt_storage_piwigo_conflict_actions_t conflict_action;
+  char server[MAX_SERVER_NAME_SIZE];
+} dt_storage_piwigo_preset_data_t;
+
 typedef struct dt_storage_piwigo_params_t
 {
+  dt_storage_piwigo_preset_data_t preset_data;
   _piwigo_api_context_t *api;
   int64_t album_id;
   int64_t parent_album_id;
   char *album;
   gboolean new_album;
-  int privacy;
-  gboolean export_tags; // deprecated - let here not to change params
-                        // size. to be removed on next version change
   gchar *tags;
+  dt_variables_params_t *vp;
 } dt_storage_piwigo_params_t;
 
-dt_storage_piwigo_conflict_actions_t conflict_action;
+void *legacy_params(dt_imageio_module_storage_t *self,
+                    const void *const old_params,
+                    const size_t old_params_size,
+                    const int old_version,
+                    int *new_version,
+                    size_t *new_size)
+{
+  if(old_version == 1)
+  {
+    // version 1 did not save any piwigo settings in the preset,
+    // so we only need to initialize the data struct here
+    typedef struct dt_storage_piwigo_preset_data_v2_t
+    {
+      char filename_pattern[DT_MAX_PATH_FOR_PARAMS];
+      dt_storage_piwigo_permissions_t privacy;
+      dt_storage_piwigo_conflict_actions_t conflict_action;
+    } dt_storage_piwigo_preset_data_v2_t;
+
+    dt_storage_piwigo_preset_data_v2_t *n =
+      (dt_storage_piwigo_preset_data_v2_t *)g_malloc0(sizeof(dt_storage_piwigo_preset_data_v2_t));
+
+    n->filename_pattern[0] = '\0';
+    n->privacy = DT_PIWIGO_PERMISSION_EVERYONE;
+    n->conflict_action = DT_PIWIGO_CONFLICT_SKIP;
+
+    *new_size = sizeof(dt_storage_piwigo_preset_data_v2_t);
+    *new_version = 2;
+
+    return n;
+  }
+  else if(old_version == 2)
+  {
+    // add server name to params
+    typedef struct dt_storage_piwigo_preset_data_v3_t
+    {
+      char filename_pattern[DT_MAX_PATH_FOR_PARAMS];
+      dt_storage_piwigo_permissions_t privacy;
+      dt_storage_piwigo_conflict_actions_t conflict_action;
+      char server[MAX_SERVER_NAME_SIZE];
+    } dt_storage_piwigo_preset_data_v3_t;
+
+    dt_storage_piwigo_preset_data_v3_t *n =
+      (dt_storage_piwigo_preset_data_v3_t *)g_malloc0(sizeof(dt_storage_piwigo_preset_data_v3_t));
+
+    memcpy(n, old_params, old_params_size);
+    n->server[0] = '\0';
+
+    *new_size = sizeof(dt_storage_piwigo_preset_data_v3_t);
+    *new_version = 3;
+
+    return n;
+  }
+
+  return NULL;
+}
 
 static char *_get_filename(const dt_image_t *img,
                            dt_imageio_module_format_t *format,
@@ -538,7 +613,7 @@ static void _piwigo_authenticate(dt_storage_piwigo_gui_data_t *ui)
       const gchar *errormessage =
         json_object_get_string_member(ui->api->response, "message");
       dt_print(DT_DEBUG_ALWAYS,
-               "[imageio_storage_piwigo] could not authenticate: `%s'!\n",
+               "[imageio_storage_piwigo] could not authenticate: `%s'!",
                errormessage);
       _piwigo_set_status(ui, _("not authenticated"), "#e07f7f");
       _piwigo_ctx_destroy(&ui->api);
@@ -549,6 +624,8 @@ static void _piwigo_authenticate(dt_storage_piwigo_gui_data_t *ui)
     _piwigo_set_status(ui, _("not authenticated, cannot reach server"), "#e07f7f");
     _piwigo_ctx_destroy(&ui->api);
   }
+
+  DT_CONTROL_SIGNAL_RAISE(DT_SIGNAL_IMAGEIO_STORAGE_EXPORT_ENABLE);
 }
 
 static void _piwigo_entry_changed(GtkEntry *entry,
@@ -557,9 +634,11 @@ static void _piwigo_entry_changed(GtkEntry *entry,
   dt_storage_piwigo_gui_data_t *ui = (dt_storage_piwigo_gui_data_t *)data;
 
   _piwigo_set_status(ui, _("not authenticated"), "#e07f7f");
-    gtk_widget_set_sensitive(GTK_WIDGET(ui->album_list), FALSE);
+  gtk_widget_set_sensitive(GTK_WIDGET(ui->album_list), FALSE);
 
   if(ui->api) _piwigo_ctx_destroy(&ui->api);
+
+  DT_CONTROL_SIGNAL_RAISE(DT_SIGNAL_IMAGEIO_STORAGE_EXPORT_ENABLE);
 }
 
 static void _piwigo_server_entry_changed(GtkEntry *entry,
@@ -573,6 +652,8 @@ static void _piwigo_server_entry_changed(GtkEntry *entry,
     _piwigo_ctx_destroy(&ui->api);
     gtk_widget_set_sensitive(GTK_WIDGET(ui->album_list), FALSE);
   }
+
+  DT_CONTROL_SIGNAL_RAISE(DT_SIGNAL_IMAGEIO_STORAGE_EXPORT_ENABLE);
 }
 
 static void _piwigo_account_changed(GtkComboBox *cb,
@@ -633,8 +714,8 @@ static void _piwigo_conflict_changed(GtkWidget *widget,
 }
 
 /** Refresh albums */
-static void _piwigo_refresh_albums(dt_storage_piwigo_gui_data_t *ui,
-                                   const gchar *select_album)
+static gboolean _piwigo_refresh_albums(dt_storage_piwigo_gui_data_t *ui,
+                                       const gchar *select_album)
 {
   gtk_widget_set_sensitive(GTK_WIDGET(ui->album_list), FALSE);
   gtk_widget_set_sensitive(GTK_WIDGET(ui->parent_album_list), FALSE);
@@ -642,7 +723,7 @@ static void _piwigo_refresh_albums(dt_storage_piwigo_gui_data_t *ui,
   if(ui->api == NULL || ui->api->authenticated == FALSE)
   {
     _piwigo_authenticate(ui);
-    if(ui->api == NULL || !ui->api->authenticated) return;
+    if(ui->api == NULL || !ui->api->authenticated) return FALSE;
   }
 
   int index = 0;
@@ -673,7 +754,7 @@ static void _piwigo_refresh_albums(dt_storage_piwigo_gui_data_t *ui,
     JsonArray *albums = json_object_get_array_member(result, "categories");
 
     const int album_len = json_array_get_length(albums);
-    
+
     if(album_len > 0 && index == 0)
       index = 1;
     if(index > album_len - 1)
@@ -724,6 +805,8 @@ static void _piwigo_refresh_albums(dt_storage_piwigo_gui_data_t *ui,
   gtk_widget_set_sensitive(GTK_WIDGET(ui->parent_album_list), TRUE);
   dt_bauhaus_combobox_set(ui->album_list, index);
   dt_bauhaus_combobox_set(ui->parent_album_list, 0);
+
+  return TRUE;
 }
 
 
@@ -739,7 +822,7 @@ static gboolean _piwigo_api_create_new_album(dt_storage_piwigo_params_t *p)
     snprintf(pid, sizeof(pid), "%"PRId64, p->parent_album_id);
     args = _piwigo_query_add_arguments(args, "parent", pid);
   }
-  args = _piwigo_query_add_arguments(args, "status", p->privacy==0?"public":"private");
+  args = _piwigo_query_add_arguments(args, "status", p->preset_data.privacy==0?"public":"private");
 
   _piwigo_api_post(p->api, args, NULL, FALSE);
 
@@ -878,7 +961,7 @@ static gboolean _piwigo_api_upload_photo(dt_storage_piwigo_params_t *p,
   // upload picture
 
   snprintf(cat, sizeof(cat), "%"PRId64, p->album_id);
-  snprintf(privacy, sizeof(privacy), "%d", p->privacy);
+  snprintf(privacy, sizeof(privacy), "%d", p->preset_data.privacy);
   snprintf(pwg_image_id_string, sizeof(pwg_image_id_string), "%d", pwg_image_id);
 
   args = _piwigo_query_add_arguments(args, "method", "pwg.images.addSimple");
@@ -909,14 +992,21 @@ static gboolean _piwigo_api_upload_photo(dt_storage_piwigo_params_t *p,
 }
 
 // Login button pressed...
-static void _piwigo_login_clicked(GtkButton *button, gpointer data)
+static void _piwigo_login_clicked(GtkButton *button, dt_imageio_module_storage_t *self)
 {
-  dt_storage_piwigo_gui_data_t *ui = (dt_storage_piwigo_gui_data_t *)data;
+  storage_login(self);
+}
+
+gboolean storage_login(dt_imageio_module_storage_t *self)
+{
+  dt_storage_piwigo_gui_data_t *ui = self->gui_data;
   _piwigo_ctx_destroy(&ui->api);
 
   gchar *last_album = dt_conf_get_string("storage/piwigo/last_album");
-  _piwigo_refresh_albums(ui, last_album);
+  gboolean res = _piwigo_refresh_albums(ui, last_album);
   g_free(last_album);
+
+  return res;
 }
 
 // Refresh button pressed...
@@ -934,18 +1024,21 @@ const char *name(const struct dt_imageio_module_storage_t *self)
   return _("Piwigo");
 }
 
+static void _filname_pattern_entry_changed_callback(GtkEntry *entry,
+                                                    gpointer user_data)
+{
+  dt_conf_set_string("plugins/imageio/storage/export/piwigo/filename_pattern", gtk_entry_get_text(entry));
+}
+
 void gui_init(dt_imageio_module_storage_t *self)
 {
-  self->gui_data =
-    (dt_storage_piwigo_gui_data_t *)g_malloc0(sizeof(dt_storage_piwigo_gui_data_t));
+  self->gui_data = g_malloc0(sizeof(dt_storage_piwigo_gui_data_t));
   dt_storage_piwigo_gui_data_t *ui = self->gui_data;
 
   ui->albums = NULL;
   ui->accounts = NULL;
   ui->api = NULL;
 
-  self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
-
   _piwigo_load_account(ui);
 
   gchar *server = dt_conf_get_string("plugins/imageio/storage/export/piwigo/server");
@@ -953,8 +1046,6 @@ void gui_init(dt_imageio_module_storage_t *self)
   // look for last server information
   _piwigo_account_t *last_account = _piwigo_get_account(ui, server);
 
-  GtkWidget *hbox, *label, *button;
-
   // account
   ui->account_list = dt_bauhaus_combobox_new_action(DT_ACTION(self));
   dt_bauhaus_widget_set_label(ui->account_list, NULL, N_("accounts"));
@@ -969,10 +1060,8 @@ void gui_init(dt_imageio_module_storage_t *self)
   gtk_widget_set_hexpand(ui->account_list, TRUE);
   g_signal_connect(G_OBJECT(ui->account_list), "value-changed",
                    G_CALLBACK(_piwigo_account_changed), (gpointer)ui);
-  gtk_box_pack_start(GTK_BOX(self->widget), ui->account_list, FALSE, FALSE, 0);
 
   // server
-  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
   ui->server_entry = GTK_ENTRY
     (dt_action_entry_new
      (DT_ACTION(self),
@@ -985,13 +1074,9 @@ void gui_init(dt_imageio_module_storage_t *self)
       last_account ? last_account->server : "piwigo.com"));
 
   gtk_widget_set_hexpand(GTK_WIDGET(ui->server_entry), TRUE);
-  gtk_box_pack_start(GTK_BOX(hbox), dt_ui_label_new(_("server")), FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(ui->server_entry), TRUE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(hbox), TRUE, TRUE, 0);
   g_free(server);
 
   // login
-  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
   ui->user_entry = GTK_ENTRY
     (dt_action_entry_new
      (DT_ACTION(self),
@@ -1002,12 +1087,8 @@ void gui_init(dt_imageio_module_storage_t *self)
       last_account ? last_account->username : ""));
 
   gtk_widget_set_hexpand(GTK_WIDGET(ui->user_entry), TRUE);
-  gtk_box_pack_start(GTK_BOX(hbox), dt_ui_label_new(_("user")), FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(ui->user_entry), TRUE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(hbox), TRUE, TRUE, 0);
 
   // password
-  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
   ui->pwd_entry =
     GTK_ENTRY(dt_action_entry_new(DT_ACTION(self),
                                   N_("password"),
@@ -1015,26 +1096,21 @@ void gui_init(dt_imageio_module_storage_t *self)
                                   last_account ? last_account->password : ""));
   gtk_entry_set_visibility(GTK_ENTRY(ui->pwd_entry), FALSE);
   gtk_widget_set_hexpand(GTK_WIDGET(ui->pwd_entry), TRUE);
-  gtk_box_pack_start(GTK_BOX(hbox), dt_ui_label_new(_("password")), FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(ui->pwd_entry), TRUE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(hbox), TRUE, TRUE, 0);
 
   // login button
-  button = gtk_button_new_with_label(_("login"));
-  gtk_widget_set_tooltip_text(button, _("Piwigo login"));
-  g_signal_connect(G_OBJECT(button), "clicked",
-                   G_CALLBACK(_piwigo_login_clicked), (gpointer)ui);
-  gtk_box_pack_start(GTK_BOX(self->widget), button, FALSE, FALSE, 0);
+  GtkWidget *login_button = gtk_button_new_with_label(_("login"));
+  gtk_widget_set_tooltip_text(login_button, _("Piwigo login"));
+  g_signal_connect(G_OBJECT(login_button), "clicked",
+                   G_CALLBACK(_piwigo_login_clicked), self);
 
   // status area
   ui->status_label = GTK_LABEL(gtk_label_new(NULL));
   gtk_label_set_ellipsize(ui->status_label, PANGO_ELLIPSIZE_END);
   gtk_widget_set_halign(GTK_WIDGET(ui->status_label), GTK_ALIGN_START);
-  gtk_box_pack_start(GTK_BOX(self->widget),
-                     GTK_WIDGET(ui->status_label), FALSE, FALSE, 0);
 
   // select account
-  if(account_index != -1) dt_bauhaus_combobox_set(ui->account_list, account_index);
+  if(account_index != -1)
+    dt_bauhaus_combobox_set(ui->account_list, account_index);
 
   // permissions list
   DT_BAUHAUS_COMBOBOX_NEW_FULL(ui->permission_list, self, NULL, N_("visible to"), NULL,
@@ -1044,54 +1120,53 @@ void gui_init(dt_imageio_module_storage_t *self)
                                N_("friends"),
                                N_("family"),
                                N_("you"));
-  gtk_box_pack_start(GTK_BOX(self->widget), ui->permission_list, FALSE, FALSE, 0);
 
   // album list
-  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
-
   ui->album_list = dt_bauhaus_combobox_new_action(DT_ACTION(self)); // Available albums
   dt_bauhaus_widget_set_label(ui->album_list, NULL, N_("album"));
+  gtk_widget_set_hexpand(GTK_WIDGET(ui->album_list), TRUE);
   g_signal_connect(G_OBJECT(ui->album_list), "value-changed",
                    G_CALLBACK(_piwigo_album_changed), (gpointer)ui);
   gtk_widget_set_sensitive(ui->album_list, FALSE);
-  gtk_box_pack_start(GTK_BOX(hbox), ui->album_list, TRUE, TRUE, 0);
 
-  button = dtgtk_button_new(dtgtk_cairo_paint_refresh, CPF_NONE, NULL);
-  gtk_widget_set_tooltip_text(button, _("refresh album list"));
-  g_signal_connect(G_OBJECT(button), "clicked",
+  GtkWidget *refresh_button = dtgtk_button_new(dtgtk_cairo_paint_refresh, CPF_NONE, NULL);
+  gtk_widget_set_tooltip_text(refresh_button, _("refresh album list"));
+  g_signal_connect(G_OBJECT(refresh_button), "clicked",
                    G_CALLBACK(_piwigo_refresh_clicked), (gpointer)ui);
-  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-
-  gtk_box_pack_start(GTK_BOX(self->widget), hbox, FALSE, FALSE, 0);
 
   // new album
-  ui->create_box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_VERTICAL, 0));
-  gtk_widget_set_no_show_all(GTK_WIDGET(ui->create_box), TRUE);
-  gtk_box_pack_start(GTK_BOX(self->widget),
-                     GTK_WIDGET(ui->create_box), FALSE, FALSE, 0);
-
-  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
-
-  label = gtk_label_new(_("title"));
-  g_object_set(G_OBJECT(label), "xalign", 0.0, (gchar *)0);
-  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-
   ui->new_album_entry = GTK_ENTRY(gtk_entry_new()); // Album title
+  gtk_widget_set_hexpand(GTK_WIDGET(ui->new_album_entry), TRUE);
   gtk_entry_set_text(ui->new_album_entry, _("new album"));
-  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(ui->new_album_entry), TRUE, TRUE, 0);
   gtk_entry_set_width_chars(GTK_ENTRY(ui->new_album_entry), 0);
 
-  gtk_box_pack_start(ui->create_box, hbox, FALSE, FALSE, 0);
-
   // parent album list
   ui->parent_album_list =
     dt_bauhaus_combobox_new_action(DT_ACTION(self)); // Available albums
   dt_bauhaus_widget_set_label(ui->parent_album_list, NULL, N_("parent album"));
   gtk_widget_set_sensitive(ui->parent_album_list, TRUE);
-  gtk_box_pack_start(ui->create_box, ui->parent_album_list, TRUE, TRUE, 0);
+
+  ui->create_box = GTK_BOX(dt_gui_vbox
+    (dt_gui_hbox(dt_ui_label_new(_("title")), ui->new_album_entry),
+     ui->parent_album_list));
+  gtk_widget_set_no_show_all(GTK_WIDGET(ui->create_box), TRUE);
 
   _piwigo_set_status(ui, _("click login button to start"), "#ffffff");
 
+  // filename pattern
+  ui->filename_pattern_entry =
+    GTK_ENTRY(dt_action_entry_new
+              (DT_ACTION(self),
+               N_("filename pattern"), G_CALLBACK(_filname_pattern_entry_changed_callback),
+               self,
+               _("enter the filename pattern for the exported images\n"
+                 "leave empty to use the image filename\n"
+                 "variables support bash like string manipulation\n"
+                 "type '$(' to activate the completion and see the list of variables"),
+               dt_conf_get_string_const("plugins/imageio/storage/export/piwigo/filename_pattern")));
+  dt_gtkentry_setup_variables_completion(ui->filename_pattern_entry);
+  gtk_editable_set_position(GTK_EDITABLE(ui->filename_pattern_entry), -1);
+
   // action on conflict
   ui->conflict_action = dt_bauhaus_combobox_new(NULL);
   dt_bauhaus_widget_set_label(ui->conflict_action, NULL, N_("on conflict"));
@@ -1099,10 +1174,23 @@ void gui_init(dt_imageio_module_storage_t *self)
   dt_bauhaus_combobox_add(ui->conflict_action, _("skip"));
   dt_bauhaus_combobox_add(ui->conflict_action, _("update metadata"));
   dt_bauhaus_combobox_add(ui->conflict_action, _("overwrite"));
-  gtk_box_pack_start(GTK_BOX(self->widget), ui->conflict_action, FALSE, FALSE, 0);
   g_signal_connect(G_OBJECT(ui->conflict_action), "value-changed",
                    G_CALLBACK(_piwigo_conflict_changed), self);
   dt_bauhaus_combobox_set(ui->conflict_action, dt_conf_get_int("storage/piwigo/conflict"));
+
+  self->widget = dt_gui_vbox
+    (ui->account_list,
+     dt_gui_hbox(dt_ui_label_new(_("server")), ui->server_entry),
+     dt_gui_hbox(dt_ui_label_new(_("user")), ui->user_entry),
+     dt_gui_hbox(dt_ui_label_new(_("password")), ui->pwd_entry),
+     login_button, ui->status_label, ui->permission_list,
+     dt_gui_hbox(ui->album_list, refresh_button),
+     ui->create_box,
+     dt_gui_hbox(dt_ui_label_new(_("filename pattern")), ui->filename_pattern_entry),
+     ui->conflict_action);
+
+  if(dt_conf_get_bool("plugins/imageio/storage/export/auto_login"))
+    storage_login(self);
 }
 
 void gui_cleanup(dt_imageio_module_storage_t *self)
@@ -1114,7 +1202,8 @@ void gui_reset(dt_imageio_module_storage_t *self)
 {
 }
 
-static uint64_t _piwigo_album_id(const gchar *name, GList *albums)
+static uint64_t _piwigo_album_id(const gchar *name,
+                                 GList *albums)
 {
   uint64_t id = 0;
 
@@ -1135,8 +1224,13 @@ static gboolean _finalize_store(gpointer user_data)
 {
   dt_storage_piwigo_gui_data_t *g = (dt_storage_piwigo_gui_data_t *)user_data;
 
-  // notify that uploads are completed to empty the lounge
+  if(g->api == NULL)
+  {
+    // not logged in, nothing to cleanup
+    return FALSE;
+  }
 
+  // notify that uploads are completed to empty the lounge
   if(!g->api->error_occured)
   {
     GList *args = NULL;
@@ -1184,6 +1278,15 @@ int store(dt_imageio_module_storage_t *self,
           dt_export_metadata_t *metadata)
 {
   dt_storage_piwigo_gui_data_t *ui = self->gui_data;
+  dt_storage_piwigo_params_t *p = (dt_storage_piwigo_params_t *)sdata;
+
+  if(p->api == NULL)
+  {
+    dt_print(DT_DEBUG_ALWAYS,
+             "[imageio_storage_piwigo] not logged in to Piwigo server!");
+    dt_control_log(_("not logged in to Piwigo server!"));
+    return 1;
+  }
 
   gint result = 0;
   gint skipped = 0;
@@ -1194,15 +1297,30 @@ int store(dt_imageio_module_storage_t *self,
   char *description = NULL;
   char *author = NULL;
 
-  dt_image_t *img = dt_image_cache_get(darktable.image_cache, imgid, 'r');
+  dt_image_t *img = dt_image_cache_get(imgid, 'r');
 
   char *filename = _get_filename(img, format, fdata);
+
+  if(*(p->preset_data.filename_pattern))
+  {
+    p->vp->filename = filename;
+    p->vp->jobcode = "export";
+    p->vp->imgid = imgid;
+    p->vp->sequence = num;
+
+    gchar *result_filename = dt_variables_expand(p->vp, p->preset_data.filename_pattern, TRUE);
+    g_free(filename);
+    filename = g_strdup(result_filename);
+    g_free(result_filename);
+  }
+
   gchar *fname = g_strconcat(darktable.tmpdir, "/", filename, NULL);
 
   if((metadata->flags & DT_META_METADATA) && !(metadata->flags & DT_META_CALCULATED))
   {
     // If title is not existing, then use the filename without
     // extension. If not, then use title instead
+    dt_pthread_mutex_lock(&darktable.metadata_threadsafe);
     GList *title = dt_metadata_get(img->id, "Xmp.dc.title", NULL);
     if(title != NULL)
     {
@@ -1229,18 +1347,19 @@ int store(dt_imageio_module_storage_t *self,
       author = g_strdup(auth->data);
       g_list_free_full(auth, &g_free);
     }
+    dt_pthread_mutex_unlock(&darktable.metadata_threadsafe);
   }
 
   g_free(filename);
 
-  dt_image_cache_read_release(darktable.image_cache, img);
+  dt_image_cache_read_release(img);
 
   if(dt_imageio_export(imgid, fname, format, fdata, high_quality, upscale,
                        TRUE, export_masks, icc_type, icc_filename,
                        icc_intent, self, sdata, num, total, metadata) != 0)
   {
     dt_print(DT_DEBUG_ALWAYS,
-             "[imageio_storage_piwigo] could not export to file: `%s'!\n",
+             "[imageio_storage_piwigo] could not export to file: `%s'!",
              fname);
     dt_control_log(_("could not export to file `%s'!"), fname);
     result = 1;
@@ -1249,7 +1368,6 @@ int store(dt_imageio_module_storage_t *self,
   dt_pthread_mutex_lock(&darktable.plugin_threadsafe);
   {
     gboolean status = TRUE;
-    dt_storage_piwigo_params_t *p = (dt_storage_piwigo_params_t *)sdata;
 
     if(metadata->flags & DT_META_TAG)
     {
@@ -1268,23 +1386,23 @@ int store(dt_imageio_module_storage_t *self,
     {
       int pwg_image_id = -1;
 
-      if(conflict_action != DT_PIWIGO_CONFLICT_NOTHING)
+      if(p->preset_data.conflict_action != DT_PIWIGO_CONFLICT_NOTHING)
       {
         pwg_image_id = _piwigo_api_get_image_id(p, img, format, fdata, 0);
       }
 
-      if(pwg_image_id >= 0 && conflict_action == DT_PIWIGO_CONFLICT_METADATA)
+      if(pwg_image_id >= 0 && p->preset_data.conflict_action == DT_PIWIGO_CONFLICT_METADATA)
       {
         status = _piwigo_api_set_info(p, author, caption, description, pwg_image_id);
         if(!status)
         {
           dt_print(DT_DEBUG_ALWAYS,
-                   "[imageio_storage_piwigo] could not update to Piwigo!\n");
+                   "[imageio_storage_piwigo] could not update to Piwigo!");
           dt_control_log(_("could not update to Piwigo!"));
           result = 1;
         }
       }
-      else if(pwg_image_id >= 0 && conflict_action == DT_PIWIGO_CONFLICT_SKIP)
+      else if(pwg_image_id >= 0 && p->preset_data.conflict_action == DT_PIWIGO_CONFLICT_SKIP)
       {
         skipped = 1;
       }
@@ -1295,7 +1413,7 @@ int store(dt_imageio_module_storage_t *self,
         if(!status)
         {
           dt_print(DT_DEBUG_ALWAYS,
-                   "[imageio_storage_piwigo] could not upload to Piwigo!\n");
+                   "[imageio_storage_piwigo] could not upload to Piwigo!");
           dt_control_log(_("could not upload to Piwigo!"));
           result = 1;
         }
@@ -1340,7 +1458,8 @@ cleanup:
 
 size_t params_size(dt_imageio_module_storage_t *self)
 {
-  return sizeof(int64_t);
+  // only nonsensitive data is stored in the preset
+  return sizeof(dt_storage_piwigo_preset_data_t);
 }
 
 void init(dt_imageio_module_storage_t *self)
@@ -1349,18 +1468,46 @@ void init(dt_imageio_module_storage_t *self)
 
 void *get_params(dt_imageio_module_storage_t *self)
 {
-  dt_storage_piwigo_gui_data_t *ui = (dt_storage_piwigo_gui_data_t *)self->gui_data;
+  dt_storage_piwigo_gui_data_t *ui = self->gui_data;
 
   if(!ui)
     return NULL; // gui not initialized, CLI mode
 
-  dt_storage_piwigo_params_t *p =
-    (dt_storage_piwigo_params_t *)g_malloc0(sizeof(dt_storage_piwigo_params_t));
+  dt_storage_piwigo_params_t *p = g_malloc0(sizeof(dt_storage_piwigo_params_t));
 
   if(!p)
     return NULL;
 
-  // fill d from controls in ui
+  // fill p from controls in ui
+  const char *text = dt_conf_get_string_const("plugins/imageio/storage/export/piwigo/filename_pattern");
+  g_strlcpy(p->preset_data.filename_pattern, text, sizeof(p->preset_data.filename_pattern));
+  text = dt_conf_get_string_const("plugins/imageio/storage/export/piwigo/server");
+  g_strlcpy(p->preset_data.server, text, sizeof(p->preset_data.server));
+
+  p->preset_data.conflict_action = dt_bauhaus_combobox_get(ui->conflict_action);
+
+  switch(dt_bauhaus_combobox_get(ui->permission_list))
+  {
+    case 0: // everyone
+      p->preset_data.privacy = DT_PIWIGO_PERMISSION_EVERYONE;
+      break;
+    case 1: // contacts
+      p->preset_data.privacy = DT_PIWIGO_PERMISSION_CONTACTS;
+      break;
+    case 2: // friends
+      p->preset_data.privacy = DT_PIWIGO_PERMISSION_FRIENDS;
+      break;
+    case 3: // family
+      p->preset_data.privacy = DT_PIWIGO_PERMISSION_FAMILY;
+      break;
+    default: // you / admin
+      p->preset_data.privacy = DT_PIWIGO_PERMISSION_ADMIN;
+      break;
+  }
+
+  p->vp = NULL;
+  dt_variables_params_init(&p->vp);
+
   if(ui->api && ui->api->authenticated == TRUE)
   {
     // create a new context for the import. set username/password to
@@ -1378,26 +1525,6 @@ void *get_params(dt_imageio_module_storage_t *self)
     p->album_id = 0;
     p->tags = NULL;
 
-    conflict_action = dt_bauhaus_combobox_get(ui->conflict_action);
-
-    switch(dt_bauhaus_combobox_get(ui->permission_list))
-    {
-      case 0:                // everyone
-        p->privacy = 0;
-        break;
-      case 1:                // contacts
-        p->privacy = 1;
-        break;
-      case 2:                // friends
-        p->privacy = 2;
-        break;
-      case 3:                // family
-        p->privacy = 4;
-        break;
-      default:               // you / admin
-        p->privacy = 8;
-    }
-
     if(index >= 0)
     {
       switch(index)
@@ -1418,33 +1545,24 @@ void *get_params(dt_imageio_module_storage_t *self)
           {
             // Something went wrong...
             dt_print(DT_DEBUG_ALWAYS,
-                     "Something went wrong.. album index %d = NULL\n", index - 2);
-            goto cleanup;
+                     "Something went wrong.. album index %d = NULL", index - 2);
+          }
+          else
+          {
+            p->album_id = _piwigo_album_id(p->album, ui->albums);
           }
-
-          p->album_id = _piwigo_album_id(p->album, ui->albums);
 
           if(!p->album_id)
           {
             dt_print(DT_DEBUG_ALWAYS,
-                     "[imageio_storage_piwigo] cannot find album `%s'!\n", p->album);
-            goto cleanup;
+                     "[imageio_storage_piwigo] cannot find album `%s'!", p->album);
           }
-
           break;
       }
     }
-    else
-      goto cleanup;
   }
-  else
-    goto cleanup;
 
   return p;
-
- cleanup:
-    g_free(p);
-    return NULL;
 }
 
 int set_params(dt_imageio_module_storage_t *self,
@@ -1453,9 +1571,47 @@ int set_params(dt_imageio_module_storage_t *self,
 {
   if(size != self->params_size(self))
     return 1;
-  // gui stuff not updated, as sensitive user data is not stored in
-  // the preset.  TODO: store name/hash in kwallet/etc module and get
-  // encrypted stuff from there!
+
+  dt_storage_piwigo_gui_data_t *g = self->gui_data;
+  dt_storage_piwigo_params_t *d = (dt_storage_piwigo_params_t *)params;
+
+  gtk_entry_set_text(GTK_ENTRY(g->filename_pattern_entry), d->preset_data.filename_pattern);
+  dt_bauhaus_combobox_set(g->conflict_action, d->preset_data.conflict_action);
+
+  if(dt_bauhaus_combobox_set_from_text(g->account_list, d->preset_data.server))
+  {
+    const _piwigo_account_t *account = _piwigo_get_account(g, d->preset_data.server);
+    if(account)
+    {
+      gtk_entry_set_text(g->server_entry, account->server);
+      gtk_entry_set_text(g->user_entry, account->username);
+      gtk_entry_set_text(g->pwd_entry, account->password);
+
+      // if we have a server name, do auto-login
+      if(dt_conf_get_bool("plugins/imageio/storage/export/auto_login"))
+        storage_login(self);
+    }
+  }
+
+  switch(d->preset_data.privacy)
+  {
+    case DT_PIWIGO_PERMISSION_EVERYONE: // everyone
+      dt_bauhaus_combobox_set(g->permission_list, 0);
+      break;
+    case DT_PIWIGO_PERMISSION_CONTACTS: // contacts
+      dt_bauhaus_combobox_set(g->permission_list, 1);
+      break;
+    case DT_PIWIGO_PERMISSION_FRIENDS: // friends
+      dt_bauhaus_combobox_set(g->permission_list, 2);
+      break;
+    case DT_PIWIGO_PERMISSION_FAMILY: // family
+      dt_bauhaus_combobox_set(g->permission_list, 3);
+      break;
+    default: // you / admin
+      dt_bauhaus_combobox_set(g->permission_list, 4);
+      break;
+  }
+
   return 0;
 }
 
@@ -1470,6 +1626,13 @@ gboolean supported(dt_imageio_module_storage_t *storage,
   return FALSE;
 }
 
+gboolean export_enabled(dt_imageio_module_storage_t *self)
+{
+  dt_storage_piwigo_gui_data_t *ui = self->gui_data;
+
+  return ui->api != NULL && ui->api->authenticated;
+}
+
 void free_params(dt_imageio_module_storage_t *self,
                  dt_imageio_module_data_t *params)
 {
@@ -1479,6 +1642,7 @@ void free_params(dt_imageio_module_storage_t *self,
   {
     g_free(p->album);
     g_free(p->tags);
+    dt_variables_params_destroy(p->vp);
     _piwigo_ctx_destroy(&p->api);
     free(p);
   }
